//+------------------------------------------------------------------+
//|                                               OccurrenceList.mqh |
//|                                      Copyright 2017, nicholishen |
//|                         https://www.forexfactory.com/nicholishen |
//+------------------------------------------------------------------+
#property copyright "Copyright 2017, nicholishen"
#property link      "https://www.forexfactory.com/nicholishen"
#property version   "1.00"
#property strict
#include <Arrays\ArrayObj.mqh>
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

class ValueCount;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class COccurrenceList : public CObject
  {
protected:
   double            m_max;
   double            m_min;
   double            m_step;
   int               m_threshold;
   int               m_resarr[];
   int               m_restotal;
   CArrayObj         m_arr;

public:
                     COccurrenceList();
                    ~COccurrenceList();
   ValueCount       *operator[](const int i);
   bool              ArrayIn( const double &arr[],
                              int          step_points=50,
                              int          min_count_threshold=10);
   int               Total()const;
protected:

   bool              AddCount(const double value);
   bool              Build(const double &arr[]);
   double            r2p(double val);

  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class ValueCount : public CObject
  {
public:
   double            value;
   int               count;
   virtual int Compare(const CObject *node,const int mode=0)const override
     {
      const ValueCount *other=node;
      if(mode>0)
        {
         double diff=this.value-other.value;
         if(fabs(diff) < _Point) return 0;
         if(diff < 0.0) return -1;
         return 1;
        }
      if(this.count > other.count) return -1;
      if(this.count < other.count) return 1;
      return Compare(node,1);
     }
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
COccurrenceList::COccurrenceList()
  {
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
COccurrenceList::~COccurrenceList()
  {
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
ValueCount *COccurrenceList::operator[](const int i)
  {
   return dynamic_cast<ValueCount*>(m_arr.At(i));
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool COccurrenceList::ArrayIn(const double &arr[],int step_points=50,int min_count_threshold=10)
  {
   m_threshold=min_count_threshold;
   m_step=(double)step_points*_Point;
   m_arr.Clear();
   m_arr.Sort();
   if(!Build(arr))
      return false;
   bool res = true;
   for(int i=ArraySize(arr)-1;i>=0;i--)
      res&=AddCount(arr[i]);

   for(int i=ArraySize(m_resarr)-1;i>=0;i--)
     {
      if(m_resarr[i]>=m_threshold)
        {
         ValueCount *vc=new ValueCount;
         vc.value = m_max - i * m_step;
         vc.count = m_resarr[i];
         
         m_arr.InsertSort(vc);
        }
     }
   //m_arr.Sort();
   return res;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int COccurrenceList::Total()const
  {
   return m_arr.Total();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+

bool COccurrenceList::AddCount(const double value)
  {
   int index=int((m_max-r2p(value))/m_step);
   if(index<0 || index>=m_restotal)
      return false;
   m_resarr[index]++;
   return true;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool COccurrenceList::Build(const double &arr[])
  {
   m_max = r2p(arr[ArrayMaximum(arr)]);
   m_min = r2p(arr[ArrayMinimum(arr)]);
   int n = int((m_max-m_min)/m_step) + 1;
   if(ArrayResize(m_resarr,n)!=n)
      return false;
   m_restotal=n;
   ArrayInitialize(m_resarr,0);
   return true;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double COccurrenceList::r2p(double val)
  {
   return m_step * round(val/m_step);
  }

//+------------------------------------------------------------------+
